// C Source File
// Created 2/9/01; 10:52:07 AM
//Original code from: Sample TSR by Greg Dietsche - Last Modification: 10/22/2002
//Modified by: Thomas Naeff, 16.03.2006


//-----------------------------------------------------------------------------
// Undocumented macro in TIGCC v0.95 and above: Do not create global imports
// for startup code. This macro is defined exclusively for ttstart; do not use
// it in your own programs. In ttstart, it only suppresses a lot of warnings.
//-----------------------------------------------------------------------------
#define _NO_INCLUDE_PATCH

//TODO: Set Your Event Hook's Name
//Hook Name: Must Be 8 non-zero characters! Use Spaces if necessary to fill this requirement
//You should edit this to reflect the name of your event hook...
//When uninstalling your event hook, the uninstall program will use this name in its list of
//installed TSR Event Hooks
	#define TSR_NAME "HideVar "

	#define HIDE_COMMAND 						"_hide"
	#define HIDE_COMMAND_LENGHT			5
	#define UNHIDE_COMMAND 					"_unhide"
	#define UNHIDE_COMMAND_LENGHT		7

//Usually this results in smaller code, OPTIMIZE_ROM_CALLS also makes calling rom functions faster
//than they would be normally
	#define OPTIMIZE_ROM_CALLS

//TODO: Do you need to have TIGCC detect the calc type?
//For Sample TSR, defining NO_CALC_DETECT will result in smaller code
//You should comment this out to see if smaller code is produced in your own projects...
//Often times, undefining (commenting out) NO_CALC_DETECT will result in
//smaller code if you use the so-called psuedo constants such as KEY_UP in compat.h
	#define NO_CALC_DETECT

//TODO: Select the calculators that you are compiling for...
	#define USE_TI89
	#define USE_TI92PLUS
	#define USE_V200

//Necessary Header Files
	#include <tigcclib.h>

//TODO: Add your own header files, and other information that needs a global scope
	void Install_TSR(void);
	void EventHandler(EVENT *ev);

	//void CheckHomeInput(EVENT *ev);
	void ProcessHomeScreenInput(EVENT *ev);
	void HomeInputToHistory(const char *HomeScreenText);
	TEXT_EDIT *TE_findHomeScreen(void);

//This Code needs TIGCC 0.94 beta 19 at the very least to function properly
#if __TIGCC__==0 && __TIGCC_MINOR__< 94 && __TIGCC_BETA__ < 19
	#warning Sample TSR - Old TIGCC Version -- Check tigcc.ticalc.org for updates
#endif

///////////////////////
// EV_hook Header		//
/////////////////////
//The Total Length of this header is 22 bytes
//The Offset to jmp Install_TSR:l is 16 bytes
//The Offset to gpOldHandler is 12 bytes
//Here is now the patch for TIGCC 0.94 from Kevin Kovler (added by Thomas Naeff)
asm(".section _stl1 | First library startup section. This is what the executable starts with.
		.ascii \"evHk" TSR_NAME "\"
		.globl gpOldHandler
		gpOldHandler:
		.long 0
		__jmp__Event_Hook__:
		jmp Install_TSR:l

		.global __calculator
		__calculator:
		.word 0");

	//Final offset: 22 bytes
///////////////////////////////
// End of EV_hook Header		//
/////////////////////////////

//Global Variables Go Here
extern EVENT_HANDLER gpOldHandler; 		//a pointer to a pointer to the old handler
char UserInput[20];
int InputCount;

//Install Function -  this function will be executed ONCE, and ONLY ONCE
void Install_TSR(void)
{
		extern char __jmp__Event_Hook__;
		char *ptr=&__jmp__Event_Hook__;

	//Set EventHandler as the default entry point from now on
	//instead of Install_TSR (this function)
		*(unsigned long*)&ptr[2]=(unsigned long)EventHandler;

	#ifndef NO_CALC_DETECT
		asm volatile("
		/* Start Of Calculator Detection Code -- From TIGCC Sources */
				move.l 0xC8,%%a0
				/* Calculator Detection */
				moveq #1,%%d0
				move.l %%a0,%%d1
				and.l #0x400000,%%d1
				jbne __calc_in_d0__
				clr.w %%d0
				move.l (%%a0,0x2F*4),%%a1 /* ScrRect */
				cmp.b #200,2(%%a1)
				jbcs __calc_in_d0__
				moveq #3,%%d0
			__calc_in_d0__:
				lea __calculator(%%pc),%%a1
				move.w %%d0,(%%a1)
		/* End Of Calculator Detection Code -- From TIGCC Sources */"
		:
		:
		: "a0", "a1", "d1");
	#endif

		//TODO: Add any one time initialization code here
		//Warning, what ever you write here must be absolutely
		//crash/error proof!!
}

void EventHandler(EVENT *ev)
{
//////////////////////////////////////////////
//Prepatory Setup for the Event Handler    //
////////////////////////////////////////////

// a special variable: pointer to a function
	EVENT_HANDLER OldHook;

	#ifdef OPTIMIZE_ROM_CALLS
		void *__saved_a5__;//a variable to save the register a5 so it can be restored later
		asm("/*START: Support for OPTIMIZE_ROM_CALLS*/");
		asm("move.l %%a5,%0":"=p"(__saved_a5__));
		asm("move.l 0xC8,%a5");
		asm("/*END: Support for OPTIMIZE_ROM_CALLS*/");
	#endif

/////////////////////////////////////////////////////////////////

//////////////////////////////////////////
//Your Event Handler Code Starts Here  //
////////////////////////////////////////

//TODO: write your very own event hook code
	//The key code should NOT be just ENTER, otherwise it could cause problems
	//if _hide or _unhide is written in the Homescreen and you're executing
	//another application -> it will execute _hide or _unhide
	if(ev->extra.Key.Code == KEY_ENTER + KEY_DIAMOND)
	{
		ProcessHomeScreenInput(ev);
	}

///////////////////////////////
//End of Your Event Handler //
/////////////////////////////

///////////////////////////////////////
//Call the Old hook if there was one//
/////////////////////////////////////

//get the address of the next TSR in line to be executed NULL if there is none
  OldHook=gpOldHandler;

//Call the old hook if there is one to be called
  if(OldHook)
  {
  	asm("								/*This inline asm takes the place of OldHook(ev);
  												the difference is that a2 will also contain ev
  												as a parameter to support TEOS. Without this code,
  												you would need to use the switch -ffixed a2 when
  												compiling this example.*/
  	move.l %0,%%a2			/*TEOS expects the parameter to be in a2*/
  	move.l %%a2,-(%%sp)	/*other event hooks expect it on the stack*/
  	move.l %1,%%a0			/*Get the address to call*/
  	jsr (%%a0)					/*Call the next event hook*/
  	addq.l #4,%%sp			/*restore the stack*/
  	"::"p"(ev),"g"(OldHook):"a0","a2");
  }//end if(OldHook)

	#ifdef OPTIMIZE_ROM_CALLS
		asm("/*START: Support for OPTIMIZE_ROM_CALLS*/");
		asm("move.l %0, %%a5	/*support for OPTIMIZE_ROM_CALLS*/"::"p"(__saved_a5__));
		asm("/*END: Support for OPTIMIZE_ROM_CALLS*/");
	#endif

}

///////////////////////////////////////////////////////////////////////////
//A Small function to prove that it is possible to call other functions //
/////////////////////////////////////////////////////////////////////////
#define SIZE_OF_HomeScreenText	30
void ProcessHomeScreenInput(EVENT *ev)
{
	//ST_helpMsg("Function started...");

	SYM_ENTRY *sym = NULL;
	char HomeScreenText[SIZE_OF_HomeScreenText + 1] = {0};
	char temp_HomeScreenText[SIZE_OF_HomeScreenText + 1] = {0};
	char MsgText[400] = {0};
	char folder[9] = {0};

	//Get the homescreen text edit structure. If you need the text, use HeapDeref(TEdit->Text.h);
	TEXT_EDIT *TEdit = TE_findHomeScreen();
	strncpy(HomeScreenText, HeapDeref(TEdit->Text.h), SIZE_OF_HomeScreenText);

	//Return if the first letter is not a underline
	if (HomeScreenText[0] != '_')
		return;

	//Check if a \ is in HomeScreenText
	//If \ is not found, get the active folder and add it later in front of the filename
	if ((strstr(HomeScreenText, "\\")) == NULL)
		{
			strcpy(temp_HomeScreenText, HomeScreenText);
			//Get the active folder
			FolderGetCur(folder);
		}


//---
//HIDE  - If the command string to hide a symbol entry is found
	if ((strncmp(HomeScreenText, HIDE_COMMAND, HIDE_COMMAND_LENGHT)) == 0)
		{
			if (strlen((HomeScreenText + HIDE_COMMAND_LENGHT + 1)) < 1)
				{
					DlgMessage(TSR_NAME, "syntax for hide: _hide PATH", BT_OK, BT_CANCEL);
					ev->extra.Key.Code = KEY_ESC;
					return;
				}

			//Set the path name if there was no \ (no folder) in the path of the homescreen input
			if (strlen(folder) > 0)
				{
					memset(HomeScreenText, 0, sizeof(HomeScreenText));
					//Copy the command
					strncpy(HomeScreenText, temp_HomeScreenText, HIDE_COMMAND_LENGHT + 1);
					//put the folder in front of the symbol name
					strcat(HomeScreenText, folder);
					strcat(HomeScreenText, "\\");
					strcat(HomeScreenText, temp_HomeScreenText + HIDE_COMMAND_LENGHT + 1);
				}

			sprintf(MsgText, "Do you want to hide the symbol '%s'?\n"
												"If the symbol is hidden, you can only make it visible again with typing\n"
												"'"UNHIDE_COMMAND" %s' in the homescreen!",
												HomeScreenText + HIDE_COMMAND_LENGHT + 1, HomeScreenText + HIDE_COMMAND_LENGHT + 1);
			if ((DlgMessage(TSR_NAME" -> Hide?", MsgText, BT_OK, BT_CANCEL)) == KEY_ENTER)
				{
					//Add the text of the home screen input to the history
					HomeInputToHistory(HomeScreenText);

					//get the SYM_ENTRY of the symbol entry given with the path after the command
					if ((sym = SymFindPtr(SYMSTR(HomeScreenText + HIDE_COMMAND_LENGHT + 1), 0)) == NULL)
						{
							sprintf(MsgText, TSR_NAME": '%s' not found", HomeScreenText + HIDE_COMMAND_LENGHT + 1);
							ST_helpMsg(MsgText);
						}
					else
						{
							//hide it
							sym->flags.bits.hidden = 1;
							sprintf(MsgText, TSR_NAME": '%s' is now hidden", HomeScreenText + HIDE_COMMAND_LENGHT + 1);
							ST_helpMsg(MsgText);
						}
				}

			//Replace the key code to ESC that other processes will be stopped
			ev->extra.Key.Code = KEY_ESC;

		}
//UNHIDE - else if the command to unhide a symbol entry is found
	else if ((strncmp(HomeScreenText, UNHIDE_COMMAND, UNHIDE_COMMAND_LENGHT)) == 0)
		{
			if (strlen((HomeScreenText + UNHIDE_COMMAND_LENGHT + 1)) < 1)
				{
					DlgMessage(TSR_NAME, "syntax for unhide: _unhide PATH", BT_OK, BT_CANCEL);
					ev->extra.Key.Code = KEY_ESC;
					return;
				}

			//Set the path name if there was no \ (no folder) in the path of the homescreen input
			if (strlen(folder) > 0)
				{
					memset(HomeScreenText, 0, sizeof(HomeScreenText));
					//Copy the command
					strncpy(HomeScreenText, temp_HomeScreenText, UNHIDE_COMMAND_LENGHT + 1);
					//put the folder in front of the symbol name
					strcat(HomeScreenText, folder);
					strcat(HomeScreenText, "\\");
					strcat(HomeScreenText, temp_HomeScreenText + UNHIDE_COMMAND_LENGHT + 1);
				}

			if ((DlgMessage(TSR_NAME" -> Unhide?", HomeScreenText + UNHIDE_COMMAND_LENGHT + 1, BT_OK, BT_CANCEL)) == KEY_ENTER)
				{
					//Add the text of the home screen input to the history
					HomeInputToHistory(HomeScreenText);

					//get the SYM_ENTRY of the symbol entry given with the path after the command
					if ((sym = SymFindPtr(SYMSTR(HomeScreenText + UNHIDE_COMMAND_LENGHT + 1), 0)) == NULL)
						{
							sprintf(MsgText, TSR_NAME": '%s' not found", HomeScreenText + UNHIDE_COMMAND_LENGHT + 1);
							ST_helpMsg(MsgText);
						}
					else
						{
							//Unhide it
							sym->flags.bits.hidden = 0;
							sprintf(MsgText, TSR_NAME": '%s' is now visible", HomeScreenText + UNHIDE_COMMAND_LENGHT + 1);
							ST_helpMsg(MsgText);


						}
				}

			//Replace the key code to ESC that other processes will be stopped
			ev->extra.Key.Code = KEY_ESC;

		}
//---


	return;
}


//==============================================================================================================

void HomeInputToHistory(const char *HomeScreenText)
{
	//Save the pointer to the top of the estack
	unsigned char * ptr = top_estack;

	//Put the command with the file path into the homescreen history
	push_END_TAG();
	push_parse_text(HomeScreenText);
	HomePushEStack();

	//Restore the pointer to the top estack
	top_estack = ptr;

	return;
}


//==============================================================================================================

// -- Comment added by Thomas Naeff: --
//Got this code from Greg's ASH source code.
// -- end of added comment --

//thanks to Samuel Stearely for explaining how his tsr hook, complete,
//gets the pointer to the home screen text edit structure in an ams
//independent manner. I (Greg) have taken his code and ported it to C.

TEXT_EDIT *TE_findHomeScreen(void)
{
				// -- Comment added by Thomas Naeff: --
				//Comment from Kevin Kofler: "Note that the register keyword is useless, GCC ignores it."
				// -- end of added comment --
        register void *a=HomeExecute;

        while(*(unsigned long*)a!=(unsigned long)TE_select)
                a+=2;

        return (TEXT_EDIT*)(unsigned long)(*(unsigned short*)(a-4));
}

//==============================================================================================================


